﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace LibAppCommon
{
    public interface ILog
    {
        void Log(LogMessage message);
    }


    public enum EN_LogType
    {
        信息,
        调试,
        告警,
        错误,
        严重错误,
        异常
    }

    public class LogMessage
    {
        public string ModeName { get; set; } = "app";
        public string Message { get; set; }
        public string MessageOneLine => Message.Replace("\r\n", " ");

        public Exception Exception { get; set; }

        public string File { get; set; }
        public string FileShortName => Path.GetFileName(File);

        public int Line { get; set; }
        public string Member { get; set; }

        public EN_LogType LogType { get; set; }

        public DateTime TimeStamp { get; set; } = DateTime.Now;

        public LogMessage() { }

        public LogMessage(EN_LogType logType, string message, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            Message = message;
            File = file;
            Line = line;
            Member = member;
            LogType = logType;
        }

        public static LogMessage FromException(Exception ex, [CallerFilePath] string file = null,
            [CallerLineNumber] int line = 0, [CallerMemberName] string member = null)
        {
            LogMessage logMessage = new LogMessage(EN_LogType.异常, GetExceptionInfo(ex), file, line, member);
            logMessage.Exception = ex;
            return logMessage;
        }

        public static string GetExceptionInfo(Exception ex)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("*********异常文本******");
            sb.AppendLine("【出现时间】：" + DateTime.Now.ToString());
            if (ex != null)
            {
                sb.AppendLine("【异常类型】：" + ex.GetType().Name);
                sb.AppendLine("【异常信息】：" + ex.Message);
                sb.AppendLine("【堆栈调用】：" + ex.StackTrace);
            }
            else
            {
                // sb.AppendLine("【附加信息】：" + backStr);
            }
            sb.AppendLine("*************************");
            return sb.ToString();
        }

    }
}
