﻿using LibOfdPrinterParam;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Printing;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace WpfOfdPrinterClient
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        DispatcherTimer _wndTimer;
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            checkShowLog.IsChecked = _isShowLog;

            comboFapiaoCountPerPage.Items.Clear();
            comboFapiaoCountPerPage.ItemsSource = Enumerable.Range(1, 4);
            comboFapiaoCountPerPage.SelectedIndex = 0;

            InitPrinterName();

            _wndTimer = new DispatcherTimer();
            _wndTimer.Interval = TimeSpan.FromSeconds(1);
            _wndTimer.Tick += dtimer_Tick;
            _wndTimer.Start();
        }

        private void InitPrinterName()
        {
            LocalPrintServer printServer = new LocalPrintServer();
            PrintQueueCollection printQueuesOnLocalServer = printServer.GetPrintQueues(
                new[] { EnumeratedPrintQueueTypes.Local, EnumeratedPrintQueueTypes.Connections });

            foreach (PrintQueue printer in printQueuesOnLocalServer)
            {
                comboPrinterName.Items.Add(new PrinterInfo() { PrintQueue = printer });
            }

        }


        private void dtimer_Tick(object sender, EventArgs e)
        {
            ShowDrawSpeed();
        }

        long _ofdDrawCountPre = 0;
        void ShowDrawSpeed()
        {
            long speed = _ofdDrawCount - _ofdDrawCountPre;
            _ofdDrawCountPre = _ofdDrawCount;

            txtOfdCreateStat.Text = string.Format("打印个数: {0} 速度: {1}", _ofdDrawCount, speed);
        }

        private void btnOfdToImageConvertor_Click(object sender, RoutedEventArgs e)
        {
            string[] fileNames = GetFileNameFromDlg(true);
            if (fileNames == null)
                return;

            string ofdFileName = fileNames.First();
            string url = txtWebServerUrl.Text;

            int drawCount = 1;
            int.TryParse(txtDrawCount.Text.Trim(), out drawCount);

            OfdFileToImageAsyn(fileNames, url, drawCount, 8);
        }

        long _ofdDrawCount;
        private async void OfdFileToImageAsyn(string[] fileNames, string url,
            int drawCount, int threadCount)
        {
            if (drawCount <= 0)
                return;

            OfdPrinterParam printerParam = GetPrinterParam();

            int totalCount = drawCount;
            threadCount = Math.Min(threadCount, drawCount);

            List<Task> taskGroup = new List<Task>();

            long ofdDrawCountThis = 0;
            DateTime startTime = DateTime.Now;

            for (int i = 0; i < threadCount; i++)
            {
                Task task = Task.Run(() =>
                {
                    try
                    {
                        while (totalCount > 0)
                        {
                            lock (this)
                            {
                                _ofdDrawCount++;
                                ofdDrawCountThis++;
                                totalCount--;
                            }

                            OfdPrinterOutput ofdPrinterOutput = OfdFileToPrint(fileNames, url, printerParam);
                            DealOutputResult(ofdPrinterOutput);
                        }
                    }
                    catch (Exception ex)
                    {
                        AddLog(ex.Message);
                    }
                });

                taskGroup.Add(task);
            }

            await Task.WhenAll(taskGroup.ToArray());

            TimeSpan span = DateTime.Now - startTime;
            double speed = ofdDrawCountThis / span.TotalSeconds;

            txtOfdCreateSpeedTotal.Text = string.Format("本次 运行时长: {0}秒 速度: {1} 个/秒",
                span.TotalSeconds.ToString("f2"), speed.ToString("f2"));
        }

        private OfdPrinterOutput OfdFileToPrint(string[] fileNames,
                        string url, OfdPrinterParam printerParam)
        {
            OfdToPrintConvertor printConvertor = new OfdToPrintConvertor() { HttpServerUrl = url };

            OfdPrinterInput inputParam = new OfdPrinterInput();
            inputParam.ClientData = Guid.NewGuid().ToString();

            foreach (string ofdFileName in fileNames)
            {
                OfdPrinterFileInput ofdFileInput = new OfdPrinterFileInput();
                string fileDataBase64 = Convert.ToBase64String(System.IO.File.ReadAllBytes(ofdFileName));

                ofdFileInput.FileName = System.IO.Path.GetFileNameWithoutExtension(ofdFileName);
                ofdFileInput.OfdFileID = Guid.NewGuid().ToString();
                ofdFileInput.OfdFileDataBase64 = fileDataBase64;
                inputParam.ListOfdFileInput.Add(ofdFileInput);
            }
            inputParam.PrinterParam = printerParam;

            // string shortName = System.IO.Path.GetFileNameWithoutExtension(ofdFileName);
            DateTime start = DateTime.Now;

            OfdPrinterOutput ofdToPdfOutput = printConvertor.OfdFileToPdf(inputParam);

            foreach (OfdPrinterFileOutput file in ofdToPdfOutput.ListOfdFileOutput)
            {
                if (!file.OK)
                {
                    AddLog(string.Format("处理异常！{0}", file.Message));
                }
            }

            int ms = (int)(DateTime.Now - start).TotalMilliseconds;
            AddLog(string.Format("处理结束! 耗时:{0}毫秒", ms));

            return ofdToPdfOutput;
        }

        private OfdPrinterParam GetPrinterParam()
        {
            OfdPrinterParam printerParam = new OfdPrinterParam();
            printerParam.IsAlignCenter = checkCenter.IsChecked == true;
            printerParam.IsRotation = checkRotation.IsChecked == true;
            printerParam.IsScaleToPaper = checkScaleToPaper.IsChecked == true;

            if (int.TryParse(comboFapiaoCountPerPage.Text.Trim(), out int countPerPage))
            {
                printerParam.PageCountPerPaper = countPerPage;
            }
            else
            {
                printerParam.PageCountPerPaper = 1;
            }

            printerParam.PrinterName = comboPrinterName.Text;
            return printerParam;
        }

        private void DealOutputResult(OfdPrinterOutput ofdPrinterOutput)
        {
        }

        void AddLog(string message)
        {
            if (!_isShowLog)
                return;

            this.Dispatcher.Invoke(() => AddLogInner(message));
        }

        void AddLogInner(string message)
        {
            LogMessage2 logMessage = new LogMessage2() { Str = new string[3] };
            logMessage.Str[0] = (ListLogInfo.Items.Count + 1).ToString();
            logMessage.Str[1] = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            logMessage.Str[2] = message;
            ListLogInfo.Items.Add(logMessage);
        }

        string[] GetFileNameFromDlg(bool multiselect)
        {
            Microsoft.Win32.OpenFileDialog ofd = new Microsoft.Win32.OpenFileDialog();
            ofd.Multiselect = multiselect;
            ofd.Filter = "ofd|*.ofd;*.zip;|All Files (*.*)|*.*||";
            if (ofd.ShowDialog() == true)
            {
                return ofd.FileNames;
            }

            return null;
        }

        private void menuItemClearAll_Click(object sender, RoutedEventArgs e)
        {
            MenuItem item = sender as MenuItem;
            if (item.Tag == null)
                return;

            ListView listItem = this.FindName(item.Tag.ToString()) as ListView;
            if (listItem == null)
                return;

            ListViewHelper helper = new ListViewHelper(listItem);

            string cmdTxt = item.Header.ToString();
            helper.ListViewMenuCmd(cmdTxt);
        }

        bool _isShowLog = true;
        bool _isSaveOfdFile = true;
        private void checkShowLog_Click(object sender, RoutedEventArgs e)
        {
            _isShowLog = checkShowLog.IsChecked == true;
        }

        private void comboFapiaoCountPerPage_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int n = (int)comboFapiaoCountPerPage.SelectedValue;
            stackPaperParam.IsEnabled = n == 1;
        }
    }

    public class LogMessage2
    {
        public string[] Str { get; set; }
    }

    public class PageImageBinding
    {
        public BitmapSource OfdPageImage { get; set; }
    }

    public class ListBoxOfdBinding
    {
        public string FullFileName { get; set; }

        public List<BitmapSource> ListPageImage { get; set; }

        public int ImageIndex { get; set; }

        public float ImageScale { get; internal set; }

        public override string ToString()
        {
            string fileNameShort = System.IO.Path.GetFileName(FullFileName);
            string result = string.Format("({0})\t{1} - {2}",
                ImageIndex, fileNameShort, ImageScale.ToString("F2"));
            return result;
        }
    }

    class PrinterInfo
    {
        public PrintQueue PrintQueue { get; set; }
        public string PrintName => PrintQueue.Name;

        public override string ToString()
        {
            return PrintQueue.Name;
        }
    }
}

